from ..settings import SEO_USE_URL_SEO

from .filters import *
from .inline import *
from .utils import *

if SEO_USE_URL_SEO:
    from .url_seo import *
else:
    from .model_instance_seo import *
    from .view_seo import *



from django.contrib.sites.shortcuts import get_current_site
from django.template import Library
from django.utils.translation import to_locale, get_language

from ..settings import SEO_DEBUG_MODE
from ..mixins.models import SeoTagsMixin

__all__ = (
    'get_seo_data',
)

register = Library()


@register.inclusion_tag('seo/seo.html', takes_context=True)
def get_seo_data(context, seo):
    """
    Renders meta data for given obj, that can be
    some instance which inherits SeoTagsMixin mixin
    """
    if isinstance(seo, SeoTagsMixin):
        return seo.as_meta(context.request, SEO_DEBUG_MODE)
    request = context.request
    return {
        'request': request,
        'canonical': request.build_absolute_uri(),
        'og_locale': to_locale(get_language()),
        'site_name': get_current_site(context.request),
    }



# Generated by Django 2.1.4 on 2019-05-14 11:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0006_urlseo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='urlseo',
            name='url',
            field=models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Url'),
        ),
    ]



from django.conf import settings as django_setting

from .const import (
    DEFAULT_IMAGE_EXTENSIONS,
    DEFAULT_OBJECT_TYPES, DEFAULT_TWITTER_TYPES
)

__all__ = (
    'SEO_DEFAULT_IMAGE',
    'SEO_IMAGE_WIDTH',
    'SEO_IMAGE_HEIGHT',
    'SEO_IMAGE_EXTENSIONS',

    'SEO_MODELS',

    'SEO_VIEWS_CHOICES',

    'SEO_SITE_NAME',

    'SEO_OG_TYPES',

    'SEO_TWITTER_TYPES',

    'SEO_FB_APP_ID',

    'SEO_HTML_ADMIN_WIDGET',

    'SEO_USE_URL_SEO',

    'SEO_USE_URL_FULL_PATH',

    'SEO_DEBUG_MODE'

)

# image settings
SEO_DEFAULT_IMAGE = getattr(django_setting, 'SEO_DEFAULT_IMAGE', '')
SEO_IMAGE_WIDTH = getattr(django_setting, 'SEO_IMAGE_WIDTH', 1200)
SEO_IMAGE_HEIGHT = getattr(django_setting, 'SEO_IMAGE_HEIGHT', 630)

# allowed image extensions for seo image
SEO_IMAGE_EXTENSIONS = getattr(django_setting, 'SEO_IMAGE_EXTENSIONS', DEFAULT_IMAGE_EXTENSIONS)

# seo models to limit content type choices
SEO_MODELS = getattr(django_setting, 'SEO_MODELS', [])

# choices of views to add seo data
SEO_VIEWS_CHOICES = getattr(django_setting, 'SEO_VIEWS_CHOICES', [])

SEO_SITE_NAME = getattr(django_setting, 'SEO_SITE_NAME', None)

# choices of open graph object types
SEO_OG_TYPES = getattr(django_setting, 'SEO_OG_TYPES', DEFAULT_OBJECT_TYPES)

# choices of twitter card types
SEO_TWITTER_TYPES = getattr(django_setting, 'SEO_TWITTER_TYPES', DEFAULT_TWITTER_TYPES)

# facebook app id
SEO_FB_APP_ID = getattr(django_setting, 'SEO_FB_APP_ID', '')

SEO_HTML_ADMIN_WIDGET = getattr(django_setting, 'SEO_HTML_ADMIN_WIDGET', {})

SEO_USE_URL_SEO = getattr(django_setting, 'SEO_USE_URL_SEO', False)

SEO_USE_URL_FULL_PATH = getattr(django_setting, 'SEO_USE_URL_FULL_PATH', False)

SEO_DEBUG_MODE = getattr(django_setting, 'SEO_DEBUG_MODE', True)



from typing import Dict

from ..models.instance_based import ModelInstanceSeo
from ..models.view_based import ViewSeo
from ..services import get_url_seo

__all__ = (
    'UrlSeoMixin',
    'ViewSeoMixin',
    'ModelInstanceViewSeoMixin'
)


class UrlSeoMixin:
    """
    Mixin to add meta tags from UrlSeo, found by a current path
    """

    def get_seo(self) -> Dict[str, ViewSeo]:
        seo = get_url_seo(request=self.request)
        return {
            'seo': seo
        }

    def get_context_data(self, **kwargs) -> Dict:
        kwargs.update(self.get_seo())
        return super().get_context_data(**kwargs)


class ViewSeoMixin(UrlSeoMixin):
    """
    Mixin to add meta tags from ViewSeo, found by a given template
    """
    seo_view = ''  # type: str

    def get_seo(self) -> Dict[str, ViewSeo]:
        return {
            'seo': (
                ViewSeo
                .objects
                .filter(
                    view=self.seo_view
                )
                .first()
            )
        }


class ModelInstanceViewSeoMixin(ViewSeoMixin):
    """
    Mixin to add meta tags from ModelInstanceSeo, found by a current content object
    """
    def get_seo(self):
        return {
            'seo': (
                ModelInstanceSeo
                .objects
                .filter_by_instance(self.object)
                .first()
            )
        }



from typing import Optional

from django.db.models import Q

from .models.url_based import UrlSeo
from .utils import get_path_from_request

__all__ = (
    'get_url_seo',
)


def get_url_seo(request) -> Optional[UrlSeo]:
    path = get_path_from_request(request=request)
    objects = (
        UrlSeo.objects
        .filter(
            Q(url=path) |
            Q(is_default=True)
        )
        .order_by('is_default')
    )
    return objects.first()



from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import pgettext_lazy

from .base_fields import base_seo_fields
from .filters import ContentObjectListFilter, SeoModelsFilter
from .utils import get_admin_base_class
from .mixins import AdminRichTextFieldMixin
from ..models.instance_based import ModelInstanceSeo
from ..utils import admin_change_url

__all__ = (
    'ModelInstanceSeoAdmin',
)


@admin.register(ModelInstanceSeo)
class ModelInstanceSeoAdmin(AdminRichTextFieldMixin, get_admin_base_class()):
    """
    Model instance seo admin interface
    """
    fieldsets = (
        (None, {
            'fields': (
                'content_type',
                'object_id',
                'content_object_link',
            )
        }),
        (pgettext_lazy('Model Instance seo admin', 'Meta'), {
            'fields': base_seo_fields,
        }),
    )

    list_display = [
        'title',
        'index',
        'follow',
        'content_object_link'
    ]
    list_editable = [
        'index',
        'follow'
    ]
    list_filter = [
        'index',
        'follow',
        'content_type',
        ContentObjectListFilter,
        SeoModelsFilter
    ]
    list_per_page = 25
    readonly_fields = [
        'content_object_link'
    ]
    search_fields = [
        'title',
        'description',
        'keywords',
        'object_id'
    ]

    def content_object_link(self, obj):
        if obj.content_object:
            url = admin_change_url(obj.content_object)
            return format_html(
                '<a href="{}">{}</a>',
                url,
                obj.content_object
            )
        return '-'
    content_object_link.short_description = pgettext_lazy(
        'Model instance seo admin',
        'Content Object'
    )

    def get_queryset(self, request):
        return (
            super().get_queryset(request)
            .prefetch_related('content_object')
        )



# Generated by Django 2.1.4 on 2019-05-15 06:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0007_auto_20190514_1144'),
    ]

    operations = [
        migrations.AddField(
            model_name='urlseo',
            name='is_default',
            field=models.BooleanField(default=False, help_text='Default instance for all pages.', verbose_name='Is default'),
        ),
    ]



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import pgettext_lazy

from .base import BaseSeoModel
from ..querysets import ModelInstanceSeoQuerySet
from ..mixins.models import SeoTagsMixin
from .. import settings

__all__ = (
    'ModelInstanceSeo',
)


class ModelInstanceSeo(SeoTagsMixin, BaseSeoModel):
    """
    Seo model, connected to objects

    Attrs:
        content_type (ForeignKey): content type
        object_id (CharField): object id
        content_object (GenericForeignKey): content object
    """

    # The mandatory fields for generic relation
    content_type = models.ForeignKey(
        ContentType,
        verbose_name=pgettext_lazy("Model instance seo model", "Content Type"),
        help_text=pgettext_lazy("Model instance seo model",
                                "Please select the type (model) "
                                "for the relation, you want to build."),
        limit_choices_to={'model__in': settings.SEO_MODELS},
        on_delete=models.CASCADE
    )
    object_id = models.CharField(
        pgettext_lazy("Model instance seo model", "Object Primary Key"),
        max_length=255,
        help_text=pgettext_lazy("Model instance seo model",
                                "Please enter the ID of the related object."),
    )
    content_object = GenericForeignKey('content_type', 'object_id')

    objects = ModelInstanceSeoQuerySet.as_manager()

    class Meta:
        verbose_name = pgettext_lazy("Model instance seo model", "Model instance seo")
        verbose_name_plural = pgettext_lazy("Model instance seo model", "Model instance seo")

    def __str__(self) -> str:
        return self.title

    def is_inherits(self):
        """
        Checks that content object inherits SeoTagsMixin
        """
        return isinstance(self.content_object, SeoTagsMixin)

    def get_meta_title(self) -> str:
        """
        Return meta title
        """
        if not self.title and self.is_inherits():
            return getattr(self.content_object, 'get_meta_title')()
        return super().get_meta_title()

    def get_meta_description(self) -> str:
        """
        Return meta description
        """
        if not self.description and self.is_inherits():
            return getattr(self.content_object, 'get_meta_description')()
        return super().get_meta_description()

    def get_h1_title(self) -> str:
        """
        Return  h1 title
        """
        if not self.h1 and self.is_inherits():
            return getattr(self.content_object, 'get_h1_title')()
        return super().get_h1_title()

    def get_meta_image_field(self):
        """
        Return image field instance to get image url
        """
        field = getattr(self, self.SEO_IMAGE_FIELD, None)
        if not field:
            field = getattr(self.content_object, self.SEO_IMAGE_FIELD, None)
        return field



from typing import Dict

from django.apps import apps
from django.contrib.sites.shortcuts import get_current_site
from django.utils.translation import to_locale, get_language

from ..settings import SEO_DEBUG_MODE
from ..mixins.models import SeoTagsMixin

__all__ = (
    'get_jinja_seo_data',
)


if apps.is_installed('django_jinja'):

    from django_jinja import library
    import jinja2

    @library.global_function
    @library.render_with("seo/jinja/seo.jinja")
    @jinja2.contextfunction
    def get_jinja_seo_data(context, obj) -> Dict[str, str]:
        """
        Renders meta data for given obj, that can be
        some instance which inherits SeoTagsMixin mixin
        """
        debug = SEO_DEBUG_MODE
        request = context['request']

        if isinstance(obj, SeoTagsMixin):
            return obj.as_meta(request, debug=debug)
        return {
            'request': request,
            'canonical': request.build_absolute_uri(),
            'og_locale': to_locale(get_language()),
            'site_name': get_current_site(request),
        }



from django.contrib import admin
from django.utils.translation import pgettext_lazy

from .base_fields import base_seo_fields
from .utils import get_admin_base_class
from .mixins import AdminRichTextFieldMixin
from ..models.view_based import ViewSeo

__all__ = (
    'ViewSeoAdmin',
)


@admin.register(ViewSeo)
class ViewSeoAdmin(AdminRichTextFieldMixin, get_admin_base_class()):
    """
    View seo admin interface
    """
    list_display = ['view', 'title', 'index', 'follow']
    list_editable = ['index', 'follow']
    list_filter = ['index', 'follow']
    search_fields = ['title', 'description', 'keywords']

    fieldsets = (
        (None, {
            'fields': ('view', )
        }),
        (pgettext_lazy('View seo admin', 'Meta'), {
            'fields': base_seo_fields,
        }),
    )



# Generated by Django 2.1.4 on 2019-05-14 09:28

import django.core.validators
from django.db import migrations, models
import seo.mixins.models
import seo.utils


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0005_auto_20190323_2228'),
    ]

    operations = [
        migrations.CreateModel(
            name='UrlSeo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_type', models.CharField(blank=True, choices=[('article', 'Article'), ('website', 'Website')], default='article', max_length=40, verbose_name='Open graph type')),
                ('twitter_type', models.CharField(blank=True, choices=[('summary', 'Summary Card'), ('summary_large_image', 'Summary Card with Large Image'), ('player', 'Player'), ('app', 'App')], default='summary', max_length=40, verbose_name='Twitter type')),
                ('index', models.CharField(blank=True, choices=[('index', 'Index'), ('noindex', 'No index')], default='index', max_length=15, verbose_name='Index robots value')),
                ('follow', models.CharField(blank=True, choices=[('follow', 'Follow'), ('nofollow', 'No follow')], default='follow', max_length=15, verbose_name='Follow robots value')),
                ('canonical', models.CharField(blank=True, max_length=255, verbose_name='Canonical')),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Seo Title')),
                ('keywords', models.TextField(blank=True, verbose_name='Meta Keywords')),
                ('description', models.TextField(blank=True, verbose_name='Meta Description')),
                ('image', models.ImageField(blank=True, null=True, upload_to=seo.utils.image_upload_to, validators=[django.core.validators.FileExtensionValidator(['jpg', 'jpeg', 'png'])], verbose_name='Image')),
                ('width', models.PositiveIntegerField(default=1200, verbose_name='Image width')),
                ('height', models.PositiveIntegerField(default=630, verbose_name='Image height')),
                ('alt', models.CharField(blank=True, max_length=255, verbose_name='Image alt text')),
                ('h1', models.CharField(blank=True, max_length=255, verbose_name='H1 title')),
                ('seo_text', models.TextField(blank=True, help_text='Can be useful for some static pages or some objects (like product category).', verbose_name='Seo text')),
                ('url', models.CharField(max_length=255, unique=True, verbose_name='Url')),
            ],
            options={
                'verbose_name': 'Url seo',
                'verbose_name_plural': 'Url seo',
            },
            bases=(seo.mixins.models.SeoTagsMixin, models.Model),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='modelinstanceseo',
            name='facebook_app_id',
        ),
        migrations.RemoveField(
            model_name='viewseo',
            name='facebook_app_id',
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='bottom_text',
            field=models.TextField(blank=True, help_text='Can be usefull for some static pages or some objects (like product category).', verbose_name='Page top text for seo'),
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='h1',
            field=models.CharField(blank=True, max_length=255, verbose_name='H1 title'),
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='top_text',
            field=models.TextField(blank=True, help_text='Can be usefull for some static pages or some objects (like product category).', verbose_name='Page top text for seo'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='bottom_text',
            field=models.TextField(blank=True, help_text='Can be usefull for some static pages or some objects (like product category).', verbose_name='Page top text for seo'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='h1',
            field=models.CharField(blank=True, max_length=255, verbose_name='H1 title'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='top_text',
            field=models.TextField(blank=True, help_text='Can be usefull for some static pages or some objects (like product category).', verbose_name='Page top text for seo'),
        ),
    ]



from django.contrib import admin
from django.utils.translation import pgettext_lazy

from .base_fields import base_seo_fields
from .utils import get_admin_base_class
from .mixins import AdminRichTextFieldMixin
from ..models.url_based import UrlSeo

__all__ = (
    'UrlSeoAdmin',
)


@admin.register(UrlSeo)
class UrlSeoAdmin(AdminRichTextFieldMixin, get_admin_base_class()):
    """
    Url seo admin interface
    """
    list_display = [
        'url',
        'title',
        'index',
        'follow',
        'is_default'
    ]
    list_editable = [
        'index',
        'follow'
    ]
    list_filter = [
        'index',
        'follow'
    ]
    search_fields = [
        'title',
        'description',
        'keywords'
    ]

    fieldsets = (
        (None, {
            'fields': (
                'url',
                'is_default'
            )
        }),
        (pgettext_lazy('Url seo admin', 'Meta'), {
            'fields': base_seo_fields,
        }),
    )



from urllib.parse import urlparse

from django.db import models
from django.utils.translation import pgettext_lazy

from .base import BaseSeoModel
from ..mixins.models import SeoTagsMixin
from .. import settings

SEO_VIEWS_CHOICES = settings.SEO_VIEWS_CHOICES

__all__ = (
    'UrlSeo',
)


class UrlSeo(SeoTagsMixin, BaseSeoModel):
    """
    Seo model to add seo staff by a url

    Attrs:
        url (CharField): url
    """

    url = models.CharField(
        pgettext_lazy("Url seo model", "Url"),
        max_length=255,
        unique=True,
        db_index=True
    )
    is_default = models.BooleanField(
        pgettext_lazy("Url seo model", "Is default"),
        default=False,
        help_text=pgettext_lazy(
            "Url seo model",
            "Default instance for all pages."
        ),
    )
    path = property(lambda self: urlparse(self.url).path)

    class Meta:
        verbose_name = pgettext_lazy("Url seo model", "Url seo")
        verbose_name_plural = pgettext_lazy("Url seo model", "Url seo")

    def __str__(self) -> str:
        return self.title

    def save(self, *args, **kwargs):
        if self.is_default:
            (
                self.__class__.objects
                .filter(is_default=True)
                .update(is_default=False)
            )
        return super().save(*args, **kwargs)



from django.contrib.contenttypes.models import ContentType
from django.db.models import QuerySet

from .settings import SEO_MODELS

__all__ = (
    'ModelInstanceSeoQuerySet',
)


class ModelInstanceSeoQuerySet(QuerySet):
    """
    QuerySet to deal with ModelInstanceSeo instances
    """

    def filter_by_instance(self, instance):
        """
        Return objects for given instance
        """
        content_type = ContentType.objects.get_for_model(instance)
        if content_type.model not in SEO_MODELS:
            return self.none()
        return self.filter(content_type=content_type, object_id=instance.pk).distinct()



from django.conf import settings
from django.urls import reverse
from django.utils.timezone import localtime, now
from django.utils.text import slugify
from django.utils.translation.trans_real import language_code_prefix_re

from .settings import SEO_USE_URL_FULL_PATH

__all__ = (
    'image_upload_to',
    'admin_change_url',
    'get_path_from_request'
)


def image_upload_to(instance, filename: str) -> str:
    """
    Util to set upload_to path,
    based on model's class name and date of uploading for image field
    """
    tz_now = localtime(now()).strftime('%Y/%m/%d')
    return f"{slugify(instance.__class__.__name__)}/{tz_now}/{filename}"


def admin_change_url(obj) -> str:
    """
    Return admin change url for given object
    """
    app_label = obj._meta.app_label
    model_name = obj._meta.model.__name__.lower()
    return reverse(f'admin:{app_label}_{model_name}_change', args=(obj.pk,))


def get_path_from_request(request, full_path: bool = SEO_USE_URL_FULL_PATH):
    """
    Return current path from request, excluding language code
    """
    if full_path:
        path = request.get_full_path()
    else:
        path = request.path

    regex_match = language_code_prefix_re.match(path)

    if regex_match:
        lang_code = regex_match.group(1)
        languages = [
            language_tuple[0] for
            language_tuple in settings.LANGUAGES
        ]
        if lang_code in languages:
            path = path[1 + len(lang_code):]
            if not path.startswith('/'):
                path = '/' + path

    return path



# Generated by Django 2.1.4 on 2019-03-15 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0003_auto_20190121_0925'),
    ]

    operations = [
        migrations.AddField(
            model_name='modelinstanceseo',
            name='canonical',
            field=models.CharField(blank=True, max_length=255, verbose_name='Canonical'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='canonical',
            field=models.CharField(blank=True, max_length=255, verbose_name='Canonical'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='alt',
            field=models.CharField(blank=True, max_length=255, verbose_name='Image alt text'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='seo_text',
            field=models.TextField(blank=True, help_text='Can be useful for some static pages or some objects (like product category).', verbose_name='Seo text'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='alt',
            field=models.CharField(blank=True, max_length=255, verbose_name='Image alt text'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='seo_text',
            field=models.TextField(blank=True, help_text='Can be useful for some static pages or some objects (like product category).', verbose_name='Seo text'),
        ),
    ]



from setuptools import setup, find_packages

pkj_name = 'seo'

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    "Framework :: Django :: 1.11",
    "Framework :: Django :: 2.0",
    "Framework :: Django :: 2.1",
    "Framework :: Django :: 2.2",
]


setup(
    name='django-ok-seo',
    version='0.6',
    description='Django seo app',
    long_description=open('README.rst').read(),
    long_description_content_type='text/x-rst',
    author='Oleg Kleschunov',
    author_email='igorkleschunov@gmail.com',
    url='https://github.com/LowerDeez/ok-seo',
    packages=[pkj_name] + [pkj_name + '.' + x for x in find_packages(pkj_name)],
    include_package_data=True,
    license='MIT',
    classifiers=CLASSIFIERS
)



__all__ = (
    'base_seo_fields',
)


base_seo_fields = (
    'object_type',
    'twitter_type',
    'index',
    'follow',
    'canonical',
    'title',
    'keywords',
    'description',
    'image',
    'width',
    'height',
    'alt',
    'h1',
    'seo_text'
)



# Generated by Django 2.1.2 on 2019-03-23 20:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0004_auto_20190315_0939'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='modelinstanceseo',
            unique_together=set(),
        ),
    ]
